/* Project filed -- Intelligent file cache and migration system
 * auth_interface.h -- Interface definitions for the auth module.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the type and function definitions for the
 *    auth module. The auth modules offers simple authentification
 *    facilities.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: auth_interface.h,v 1.1 2000/04/10 18:40:23 schaefer Exp $
 */

#ifndef _AUTH_INTERFACE_H
#define _AUTH_INTERFACE_H

/* Type definitions */

/* Function definitions */

/* NAME
 *    auth_challenge_receive_and_respond
 * DESCRIPTION
 *    Reads a challenge from the socket/pipe/pty sd, and issues
 *    an authenticated answer.
 * RESULT
 *    TRUE if I/O process was fine.
 * NOTES
 *    - Secret is a 8 bit stream.
 * BUGS
 * TODO
 */
BOOL auth_challenge_receive_and_respond(fd_t sd,
                                        const unsigned char *secret,
                                        const int secret_length);

/* NAME
 *    auth_random_cleartext_challenge_issue_and_verify
 * DESCRIPTION
 *    Writes a challenge to the socket/pipe/pty sd, reads the answer,
 *    and verifies it is authenticated.
 * RESULT
 *    TRUE if I/O process was fine and authentification succeeded.
 * NOTES
 *    - Secret is a 8 bit stream.
 * BUGS
 * TODO
 */
BOOL auth_random_cleartext_challenge_issue_and_verify
        (fd_t sd,
         const unsigned char *secret,
         const int secret_length);


#endif /* !_AUTH_INTERFACE_H */
