/* Project filed -- Intelligent file cache and migration system
 * buffer.h -- Buffered efficient I/O module
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the implementation for the function offered
 *    by the buffer module. The buffer module offers buffered
 *    efficient I/O from and to UNIX file descriptors.
 * NOTES
 *    - Current implementation is:
 *         two UNIX processes (fork())
 *         shared memory
 *         pipes for synchronization
 *    - This resembles a lot the GPL program buffer, to which I have
 *      contributed patches in the past. However it doesn't content
 *      any `buffer' code.
 * BUGS
 * TODO
 *    - We could split this into three functions: initialize,
 *      deinitialize and do_io. This would allow to do I/O by chunks
 *      without the overhead. However it hasn't any use for now.
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: buffer_interface.h,v 1.2 2000/08/05 20:09:24 schaefer Exp $
 */

#ifndef _BUFFER_INTERFACE_H
#define _BUFFER_INTERFACE_H
 
/* Function definitions */

/* NAME
 *    buffer_do_io
 * DESCRIPTION
 *    This will do the requested I/O in a buffered fashion, until
 *    end of file or error on input, or error on output.
 * RESULT
 *    TRUE if data could be successfully transmitted, FALSE else.
 * NOTES
 *    - input and output are assumed to be valid file descriptors,
 *      of standard files, or pipes/sockets too.
 *    - If returns FALSE, the_errno is set to the actual error,
 *      and input_error is set to TRUE if the error was on the input
 *      fd. Only one error is recorded.
 *    - the_errno and input_error cannot be NULL.
 * BUGS
 *    - Errors which are neither input nor output are wrongly told.
 * TODO
 */
BOOL buffer_do_io(fd_t input,
                  fd_t output,
                  int *the_errno,
                  BOOL *input_error);

#endif /* !_BUFFER_INTERFACE_H */
