/* Project filed -- Intelligent file cache and migration system
 * test.c -- Test of the buffer module
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This is a simple test program used to, well, test the buffer
 *    module.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: test.c,v 1.1 2000/08/05 10:05:43 schaefer Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "global.h"
#include "debug_interface.h"
#include "buffer_interface.h"

/* Private function definitions */

/* NAME
 *    usage
 * DESCRIPTION
 *    Displays correct program usage, and return 2.
 * RESULT
 *    (int) 2
 * NOTES
 * BUGS
 * TODO
 */
static int usage(const char *program_name);

/* NAME
 *    do_operation
 * DESCRIPTION
 *    Do the requested operation.
 * RESULT
 *    0 everything fine
 *    1 something failed.
 * NOTES
 * BUGS
 * TODO
 */
static int do_operation(const char *input_file, const char *output_file);

/* Function implementation */

int main(int argc, char **argv) {
   if (argc == 3) {
      return do_operation(argv[1], argv[2]);
   }
   else {
      return usage(argv[0]);
   }

   /* NOT REACHED */
}

/* Private function implementation */

static int do_operation(const char *input_file, const char *output_file) {
   fd_t in_fd;
   fd_t out_fd;
   int result = 0;

   in_fd = open(input_file, O_RDONLY);
   if (in_fd == -1) {
      debug_perror(DEBUG_LEVEL_ERROR, "open(in)");
      result = 1;
   }
   else {
      out_fd = open(output_file, O_WRONLY); /* no creat() */
      if (out_fd == -1) {
	 debug_perror(DEBUG_LEVEL_ERROR, "open(out)");
	 result = 1;
      }
      else {
         int the_errno;
         BOOL input_error;

         if (buffer_do_io(in_fd, out_fd, &the_errno, &input_error)) {
            debug_log(DEBUG_LEVEL_INFO, "buffered I/O succeeded.\n");
         }
         else {
            debug_log(DEBUG_LEVEL_ERROR,
                      "buffered I/O failed: %s [%d] %s.\n",
                      strerror(the_errno),
                      the_errno,
                      input_error ? "IN"
                                  : "OUT");
            result = 1;
         }

	 if (close(out_fd)) {
	    debug_perror(DEBUG_LEVEL_ERROR, "close(out)");
	    result = 1;
	 }
      }

      if (close(in_fd)) {
         debug_perror(DEBUG_LEVEL_ERROR, "close(in)");
         result = 1;
      }
   }

   return result;
}

static int usage(const char *program_name) {
   fprintf(stderr, "%s in_file out_file\n"
                   "%s: bad args.\n",
                   program_name,
                   program_name);

   return 2;
}
