/* Project filed -- Intelligent file cache and migration system
 * crypt.c -- Implementation for DES crypt glue.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This module encapsulates the public domain DES encryption routines.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: crypt.c,v 1.2 2000/04/11 18:10:45 schaefer Exp $
 */

#include <stdlib.h>
#include <string.h>
#include "global.h"
#include "crypt_interface.h"
#include "des.h"

/* Private types */

typedef struct {
   int num;
   des_cblock ivec;
} crypt_info_param_t;

typedef struct {
   crypt_info_param_t param;
   des_key_schedule schedule;
} crypt_info_t;

/* Function implementation */

void *crypt_context_allocate(void) {
   return calloc(1, sizeof(crypt_info_t));
}

BOOL crypt_context_initialize(void *context,
                              const char *key,
                              const int key_length) {
   crypt_info_t *info = (crypt_info_t *) context;
   des_cblock the_key;

   /* The author recommends using md5sum(key), taking the first 8 bytes
    * of the result instead of this. For now, this is simpler. Alternatively,
    * the the_key (8 uchar) could be random. See also
    * /usr/doc/ssleay/doc/des.txt.gz
    */
   des_string_to_key((char *) key, &the_key);
 
   des_check_key = 1; /* verify my key is not known weak */
   if (des_set_key(&the_key, info->schedule)) {
      return FALSE;
   }
   else {
      return TRUE;
   }
   /* NOT REACHED */
}

void crypt_context_free(void **context) {
   free (*context);
   *context = NULL;
}

void crypt_do(void *context,
              const unsigned char *input_buffer,
              const int input_buffer_length,
              unsigned char *output_buffer,
              BOOL initial,
              BOOL encrypt) {
   crypt_info_t *info = (crypt_info_t *) context;

   if (initial) {
      memset(&info->param, 0, sizeof(info->param));
   }

   des_cfb64_encrypt((unsigned char *) input_buffer,
                     output_buffer,
                     input_buffer_length,
                     info->schedule,
                     &info->param.ivec,
                     &info->param.num,
                     encrypt ? 0 : DES_ENCRYPT);
}

