/* Project filed -- Intelligent file cache and migration system
 * crypt_interface.h -- Interface definitions for the crypt module.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the type and function definitions for the
 *    crypt module. The crypt modules encapsulates the public domain
 *    DES crypt routines.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: crypt_interface.h,v 1.1 2000/04/10 18:40:23 schaefer Exp $
 */

#ifndef _CRYPT_INTERFACE_H
#define _CRYPT_INTERFACE_H

/* Type definitions */

/* Function definitions */

/* NAME
 *    crypt_context_allocate
 * DESCRIPTION
 *    Allocates a CRYPT context structure, to be used in later calls
 *    of crypt_context_initialize(), a few crypt_compute_hash() and
 *    then crypt_context_free().
 * RESULT
 *    void *, NULL if allocation failed.
 * NOTES
 *    - Once allocated, the context must be either crypt_context_free()d
 *      immediately, or passed first to crypt_context_initialize(), and
 *      then a number of times to crypt_compute_hash(), then to
 *      either crypt_context_initialize() and a few crypt_compute_hash(),
 *      or crypt_context_free().
 * BUGS
 * TODO
 */
void *crypt_context_allocate(void);

/* NAME
 *    crypt_context_initialize
 * DESCRIPTION
 *    Initializes a CRYPT context structure with a key.
 * RESULT
 *    FALSE if your key is *weak*, TRUE normally.
 * NOTES
 * BUGS
 *    - At this time, strlen(key) must be equal to key_length, ie
 *      no embedded \0.
 * TODO
 */
BOOL crypt_context_initialize(void *context,
                              const char *key,
                              const int key_length);

/* NAME
 *    crypt_context_free
 * DESCRIPTION
 *    Frees a CRYPT context structure previously allocated with
 *    crypt_context_allocate(). Will set context to NULL.
 * RESULT
 *    NONE
 * NOTES
 * BUGS
 * TODO
 */
void crypt_context_free(void **context);

/* NAME
 *    crypt_do
 * DESCRIPTION
 *    Crypts a buffer.
 * RESULT
 *    NONE
 * NOTES
 *    - You must allocate, initialize and free the context yourself.
 *    - Output buffer must be as big as input_buffer_length.
 *    - initial must be set if not a continuation.
 * BUGS
 * TODO
 */
void crypt_do(void *context,
              const unsigned char *input_buffer,
              const int input_buffer_length,
              unsigned char *output_buffer,
              BOOL initial,
              BOOL encrypt);

#endif /* !_CRYPT_INTERFACE_H */
