/* Project filed -- Intelligent file cache and migration system
 * debug.c -- Debug log routines
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the implementation for the function offered
 *    by the debug module. The debug module offers simple debug routines.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: debug.c,v 1.1 2000/05/20 16:02:16 schaefer Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <stdarg.h>
#include "global.h"
#include "debug_interface.h"

/* Constants */

const char *debug_level_descriptions[]
   = {
        "DEBUG", "INFO", "WARN", "ERROR", "FATAL"
     };

/* Global variables */

static debug_level_t debug_level = DEBUG_LEVEL_DEFAULT;

/* Function implementation */

void debug_log(debug_level_t level,
               const char *format,
               ...) {
   va_list p;

   va_start(p, format);

   if (level >= debug_level) {
      fprintf(stderr, "[%s] ", debug_level_description(level));
      vfprintf(stderr, format, p);
   }

   va_end(p);
}

void debug_perror(debug_level_t level, const char *str) {
  debug_log(level, "%s: %s.\n", str, strerror(errno));
}

debug_level_t debug_set_level(debug_level_t level) {
   debug_level_t old = debug_level;

   debug_level = level;

   return old;
}

const char *debug_level_description(debug_level_t level) {
   if ((level < 0)
       || (level
           >= sizeof(debug_level_descriptions) / sizeof(const char *))) {
      return "INVALID";
   }
   else {
      return debug_level_descriptions[level];
   }
   /* NOT REACHED */
}
