/* Project filed -- Intelligent file cache and migration system
 * debug_interface.h -- Interface definitions for the debug module.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the type and function definitions for the
 *    debug module. The debug module offers simple debug routines.
 * NOTES
 *    - This module has no initialization or deinitialization
 *      functions.
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: debug_interface.h,v 1.2 2000/07/07 12:53:12 schaefer Exp $
 */

#ifndef _DEBUG_INTERFACE_H
#define _DEBUG_INTERFACE_H

/* Type definitions */

typedef enum {
   DEBUG_LEVEL_GARBAGE, /* Debug information */
   DEBUG_LEVEL_INFO,    /* Progression indicators */
   DEBUG_LEVEL_WARN,    /* Condition which requires a retry, or otherwise
                         * an exceptionnal condition which isn't an
                         * error.
                         */
   DEBUG_LEVEL_ERROR,   /* Errors which do not prevent normal execution */
   DEBUG_LEVEL_FATAL    /* Error which prevents normal execution */
} debug_level_t;

#define DEBUG_LEVEL_DEFAULT DEBUG_LEVEL_GARBAGE

/* Function definitions */

/* NAME
 *    debug_log
 * DESCRIPTION
 *    Logs a string (in printf format) depending on log level.
 * RESULT
 *    NONE
 * NOTES
 * BUGS
 * TODO
 */
void debug_log(debug_level_t level,
               const char *format,
               ...);

/* NAME
 *    debug_log
 * DESCRIPTION
 *    Calls debug_log() with the error description for the last error,
 *    in the vein of perror().
 * RESULT
 *    NONE
 * NOTES
 * BUGS
 * TODO
 */
void debug_perror(debug_level_t level, const char *str);

/* NAME
 *    debug_set_level
 * DESCRIPTION
 *    Changes the debug level. The parameter is the last debug
 *    level allowed to print. For example, if you specify
 *    DEBUG_LEVEL_GARBAGE, you get *all* messages. If you specify
 *    DEBUG_LEVEL_FATAL, you only get FATAL error messages.
 * RESULT
 *    The old debug level, debug_level_t.
 * NOTES
 * BUGS
 * TODO
 */
debug_level_t debug_set_level(debug_level_t level);

/* NAME
 *    debug_level_description
 * DESCRIPTION
 *    Returns a constant string describing an error level.
 * RESULT
 *    const char *
 * NOTES
 * BUGS
 * TODO
 */
const char *debug_level_description(debug_level_t level);

#endif /* !_DEBUG_INTERFACE_H */
