/* Project filed -- Intelligent file cache and migration system
 * inet.c -- TCP/IP socket support functions.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the implementation for the function offered
 *    by the inet module. The inet modules offers simple TCP/IP
 *    server and client functionnality.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: inet.c,v 1.2 2000/04/11 18:09:19 schaefer Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>
#include <string.h>
#include "global.h"
#include "inet_interface.h"

/* Type definitions */

/* Private function definitions */

/* Function implementation */

fd_t inet_connect_to_server(const char *server_name,
                            const int port) {
   fd_t sd;

   sd = socket(AF_INET, SOCK_STREAM, 0);
   if (sd != -1) {
      struct sockaddr_in serv_addr;
      struct hostent *hp;

      serv_addr.sin_family = AF_INET;
      hp = gethostbyname(server_name);
      if (hp == NULL) {
         perror("unknown host");
         close(sd);
         sd = -1;
      }
      else {
         memcpy(&serv_addr.sin_addr, hp->h_addr, hp->h_length);
         serv_addr.sin_port = htons(port);
         if (connect(sd, &serv_addr, sizeof(serv_addr))) {
	    perror("connect()");
	    close(sd); /* res. ignored */
	    sd = -1;
	 }
      }
   }
   else {
      perror("socket()");
   }

   return sd;
}

fd_t inet_create_endpoint(const int port, BOOL reuse) {
   fd_t sd;

   sd = socket(AF_INET, SOCK_STREAM, 0);
   if (sd != -1) {
      int val = 1;
      struct sockaddr_in serv_addr;

      serv_addr.sin_family = AF_INET;
      serv_addr.sin_addr.s_addr = INADDR_ANY;
      serv_addr.sin_port = htons(port);

      if (reuse
          && setsockopt(sd,
                        SOL_SOCKET,
                        SO_REUSEADDR,
                        &val,
                        sizeof(serv_addr))) {
         perror("setsockopt(SO_REUSEADDR)");
         close(sd); /* res. ignored */
         sd = -1;
      }
      else if (bind(sd, &serv_addr, sizeof(serv_addr))) {
	 perror("bind()");
	 close(sd); /* res. ignored */
	 sd = -1;
      }
      else {
         if (listen(sd, 5)) {
	    perror("listen()");
	    close(sd); /* res. ignored */
	    sd = -1;
         }
      }
   }
   else {
      perror("socket()");
   }

   return sd;
}

/* Private function implementation */

