/* Project filed -- Intelligent file cache and migration system
 * inet_interface.h -- Interface definitions for the inet module.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the type and function definitions for the
 *    inet module. The inet modules offers simple TCP/IP socket client
 *    and server services.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: inet_interface.h,v 1.2 2000/04/11 18:09:19 schaefer Exp $
 */

#ifndef _INET_INTERFACE_H
#define _INET_INTERFACE_H

/* Type definitions */

/* Function definitions */

/* NAME
 *    inet_connect_to_server
 * DESCRIPTION
 *    This opens a connection to the server at the specificied host
 *    named and port (in local notation).
 * RESULT
 *    socket fd or -1 if any error.
 * NOTES
 *    - This is a client-only function.
 *    - The returned socket may be used with read(), write() and
 *      others, then close()d.
 * BUGS
 * TODO
 */
fd_t inet_connect_to_server(const char *server_name,
                            const int port);


/* NAME
 *    inet_create_endpoint
 * DESCRIPTION
 *    This creates a listening endpoint for client connections at
 *    the current host and given port.
 * RESULT
 *    socket fd or -1 if any error.
 * NOTES
 *    - This is a server-only function.
 *    - The returned socked must be used in accept(), then close()d.
 *    - Specify TRUE for reuse so that the socket can be reused
 *      immediately after the server dies, e.g.
 * BUGS
 * TODO
 */
fd_t inet_create_endpoint(const int port, BOOL reuse);

#endif /* !_INET_INTERFACE_H */
