/* Project filed -- Intelligent file cache and migration system
 * glue.c -- Implementation for md5 glue.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This module encapsulates the public domain MD5sum routines.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: glue.c,v 1.3 2001/02/04 12:23:18 schaefer Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include "global.h"
#include "md5_interface.h"
#include "md5sum.h"

/* Private function definitions */

/* NAME
 *    hexadecimal_to_value
 * DESCRIPTION
 *    Converts an hexadecimal two character string into a byte.
 * RESULT
 *    unsigned char
 * NOTES
 *    - String must be valid.
 * BUGS
 * TODO
 */
static unsigned char hexadecimal_to_value(unsigned char *str);

/* Function implementation */

void md5_compute_hash(void *context,
                      const unsigned char *buffer,
                      const int buffer_length,
                      md5_hash_ptr_t hash,
                      BOOL final) {
   MD5Update((struct MD5Context *) context, buffer, buffer_length);
   if (final) {
      MD5Final(hash, (struct MD5Context *) context);
   }
}

void *md5_context_allocate(void) {
   return calloc(1, sizeof(struct MD5Context));
}

void md5_context_initialize(void *context) {
   MD5Init((struct MD5Context *) context);  
}

void md5_context_free(void **context) {
   free(*context);
   *context = NULL;
}

void md5_hash_to_textual_md5sum(const md5_hash_ptr_t hash,
                                md5_hash_textual_ptr_t text) {
   unsigned int i;

   for (i = 0; i < MD5_HASH_STRING_LENGTH; i++) {
      snprintf(&text[i << 1], 3, "%2.2x", hash[i]);
   }
}

void md5_textual_md5sum_to_hash(const md5_hash_textual_ptr_t text,
                                md5_hash_ptr_t hash) {
   unsigned int i;

   for (i = 0; i < MD5_HASH_STRING_LENGTH; i++) {
      hash[i] = (hexadecimal_to_value(&text[i << 1]) * 16)
                + hexadecimal_to_value(&text[(i << 1) + 1]);
   }
}

/* Private function implementation */

static unsigned char hexadecimal_to_value(unsigned char *str) {
   if ((*str >= '0') && (*str <= '9')) {
      return (*str - '0');
   }
   else if ((*str >= 'A') && (*str <= 'Z')) {
      return (*str - 'A');
   }
   else if ((*str >= 'a') && (*str <= 'z')) {
      return (*str - 'a');
   }
   else {
      return 0;
   }
}