/* Project filed -- Intelligent file cache and migration system
 * test.c -- Quick test implementation for the md5 module
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This test software does a simple equivalent of the md5sum UNIX
 *    command, but on stdin only.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: test.c,v 1.1 2001/02/04 13:10:01 schaefer Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include "global.h"
#include "md5_interface.h"
#include "md5sum.h"

#define BUFFER_SIZE 32768

int main(int argc, char **argv) {
   if (argc == 1) {
      void *md5_context = md5_context_allocate();
      if (md5_context) {
         BOOL finished = FALSE;
         int result = 0; /* succeeds */
         unsigned char buffer[BUFFER_SIZE];

         md5_context_initialize(md5_context);

         while (!finished && (result == 0)) {
            size_t actual = fread(buffer, 1, sizeof(buffer), stdin);

	    md5_compute_hash(md5_context,
			     buffer /* in clear */, 
			     actual, /* can be zero */
			     NULL,
			     FALSE);
            finished = (actual == 0);
            if (finished && ferror(stdin)) {
               result = 1;
            }
         }

         if (result == 0) {
            unsigned char hash[MD5_HASH_STRING_LENGTH];
            char textual_hash[MD5_HASH_TEXTUAL_STRING_LENGTH];

            md5_compute_hash(md5_context, NULL, 0, hash, TRUE);

            md5_hash_to_textual_md5sum(hash, textual_hash);

            printf("%s\n", textual_hash);
         }

         md5_context_free(md5_context);

         return result;
      }
      else {
         fprintf(stderr, "%s: can't allocate context.\n", argv[0]);
         return 1;
      }
      /* NOT REACHED */
   }
   else {
      fprintf(stderr, "%s: bad args.\n", argv[0]);
      return 2;
   }
   /* NOT REACHED */
}