/* Project filed -- Intelligent file cache and migration system
 * misc.c -- Miscellaneous support functions.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the implementation for the function offered
 *    by the misc module. The misc modules offers facilities not found
 *    in the rest of the support library.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: misc.c,v 1.6 2000/08/05 20:09:45 schaefer Exp $
 */

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h> 
#include <errno.h>
#include "global.h"
#include "misc_interface.h"

ssize_t misc_all_read(fd_t fd, unsigned char *buf, size_t count) {
   ssize_t total_done = 0;
   ssize_t temp;

   do {
      temp = read(fd, buf, count);

      if (temp > 0) {
         count -= temp;
         buf += temp;
         total_done += temp;
      }
      else if (temp < 0) {
         int errno_copy = errno;

         if (errno != EAGAIN) {
            perror("mar-read()");
            errno = errno_copy;
         }
         return (-1);
      }
   } while (temp && count);

   return total_done;
}

ssize_t misc_all_write(fd_t fd, unsigned char *buf, size_t count) {
   ssize_t total_done = 0;
   ssize_t temp;

   do {
      temp = write(fd, buf, count);

      if (temp > 0) {
         count -= temp;
         buf += temp;
         total_done += temp;
      }
      else if (temp < 0) {
         int errno_copy = errno;
         if (errno != EAGAIN) {
            perror("mar-write()");
            errno = errno_copy;
         }
         return (-1);
      }
   } while (temp && count);

   return total_done;
}

BOOL misc_get_random_data(unsigned char *buf,
                          size_t count) {
   fd_t fd;
   BOOL result = TRUE; /* succeeded */

   fd = open("/dev/random", O_RDONLY);
   if (fd != -1) {
      if (misc_all_read(fd, buf, count) != count) {
         perror("misc_all_read()");
         result = FALSE;
      }

      if (close(fd)) {
         perror("close()");
         result = FALSE;
      }
   }
   else {
      perror("open(\"/dev/random\")");
      result = FALSE;
   }

   return result;
}

void misc_delete_spaces(unsigned char **buffer) {
   unsigned char *tmp;
   int last_non_space = 0;
   int index = 0;

   while (**buffer) {
      if (!isspace(**buffer)) {
         break; /* out of while */
      }
      (*buffer)++;
   }

   tmp = *buffer;

   while (*tmp) {
      if (!isspace(*tmp)) {
         last_non_space = index;
      }
      tmp++;
      index++;
   }

   (*buffer)[last_non_space + 1] = '\0'; /* at worst re-sets NUL */
}
