/* Project filed -- Intelligent file cache and migration system
 * misc_interface.h -- Interface definitions for the miscealleanous module.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains the type and function definitions for the
 *    misc module. The misc modules offers facilities not found in the
 *    rest of the support library.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: misc_interface.h,v 1.4 2000/08/05 10:08:53 schaefer Exp $
 */

#ifndef _MISC_INTERFACE_H
#define _MISC_INTERFACE_H

/* Type definitions */

/* Function definitions */

/* NAME
 *    misc_all_read
 * DESCRIPTION
 *    This function is an interface to the system read() function,
 *    with the following difference: we guarantee that count bytes
 *    will be read, or an error. Basically this function should be
 *    used for read on character devices (ttys, ptys, sockets,
 *    /dev/random, etc), where a read can return less than requested,
 *    when a specific size is expected.
 * RESULT
 *    ssize_t, in case of error reading a part, returns -1.
 * NOTES
 *    - please consult read(2)
 *    - non-blocking mode must be OFF
 *    - preserves errno so that calling program can get the error.
 * BUGS
 * TODO
 */
ssize_t misc_all_read(fd_t fd, unsigned char *buf, size_t count);

/* NAME
 *    misc_all_write
 * DESCRIPTION
 *    This function is an interface to the system write() function,
 *    with the following difference: we guarantee that count bytes
 *    will be write, or an error. Basically this function should be
 *    used for writes on character devices (ttys, ptys, sockets, etc)
 *    where a write can send less than requested, when a specific size
 *    is expected.
 * RESULT
 *    ssize_t, in case of error reading a part, returns -1.
 * NOTES
 *    - please consult write(2)
 *    - non-blocking mode must be OFF
 *    - preserves errno so that calling program can get the error.
 * BUGS
 * TODO
 */
ssize_t misc_all_write(fd_t fd, unsigned char *buf, size_t count);

/* NAME
 *    misc_get_random_data
 * DESCRIPTION
 *    Returns cryptographically safe random data. This might
 *    block for an undetermined amount of time.
 * RESULT
 *    TRUE if ok.
 * NOTES
 * BUGS
 * TODO
 */
BOOL misc_get_random_data(unsigned char *buf,
                          size_t count);

/* NAME
 *    misc_delete_spaces
 * DESCRIPTION
 *    Removes initial and final spaces, modifying the
 *    buffer pointer.
 * RESULT
 *    NONE
 * NOTES
 * BUGS
 * TODO
 */
void misc_delete_spaces(unsigned char **buffer);

#endif /* !_MISC_INTERFACE_H */

