/* Project filed -- Intelligent file cache and migration system
 * test.c -- Test of the changer/taper subsystems
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This is a simple test program used to, well, test the changer
 *    and taper subsystems. We cheat a bit.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: test.c,v 1.12 2000/08/06 09:32:47 schaefer Exp $
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "global.h"
#include "debug_interface.h"
#include "changer_interface.h"
#include "taper_interface.h"

/* Private function definitions */

/* NAME
 *    usage
 * DESCRIPTION
 *    Displays correct program usage, and return 2.
 * RESULT
 *    (int) 2
 * NOTES
 * BUGS
 * TODO
 */
static int usage(const char *program_name);

/* NAME
 *    do_show
 * DESCRIPTION
 *    Does showing of all slot's content.
 * RESULT
 *    0 if everything ok, 1 else.
 * NOTES
 * BUGS
 * TODO
 */
static int do_show(const char *program_name,
                   const char *changer_device_name,
                   const char *tape_device_name);

/* NAME
 *    do_label
 * DESCRIPTION
 *    Labels the current tape in the drive and tells the changer about it.
 * RESULT
 *    0 if everything ok, 1 else.
 * NOTES
 * BUGS
 * TODO
 */
static int do_label(const char *program_name,
                    const char *changer_device_name,
                    const char *tape_device_name,
                    const char *label,
                    const unsigned int slot);

/* NAME
 *    do_load_tape
 * DESCRIPTION
 *    Loads the tape with the specified label into the specified drive.
 * RESULT
 *    0 if everything ok, 1 else.
 * NOTES
 * BUGS
 * TODO
 */
static int do_load_tape(const char *program_name,
			const char *changer_device_name,
			const char *tape_device_name,
			const char *label);

/* NAME
 *    do_unload_tape
 * DESCRIPTION
 *    Unloads the specified drive.
 * RESULT
 *    0 if everything ok, 1 else.
 * NOTES
 * BUGS
 * TODO
 */
static int do_unload_tape(const char *program_name,
		  	  const char *changer_device_name,
			  const char *tape_device_name);

/* NAME
 *    do_read_part
 * DESCRIPTION
 *    Read a part from the specified tape to disk.
 * RESULT
 *    0  all ok
 *    1  something wrong
 * NOTES
 * BUGS
 * TODO
 */
static int do_read_part(const char *program_name,
                        const char *changer_device_name,
                        const char *tape_device_name,                       
                        const char *label,
                        const char *output_file_name,
                        const taper_file_number_t file_number);
                        
/* NAME
 *    do_write_part
 * DESCRIPTION
 *    Write a part from disk to the end of the specified tape.
 * RESULT
 *    0  all ok
 *    1  something wrong
 * NOTES
 * BUGS
 * TODO
 */
static int do_write_part(const char *program_name,
                         const char *changer_device_name,
                         const char *label,
                         const char *tape_device_name,
                         const char *input_file_name);

/* NAME
 *    is_fine_taper
 * DESCRIPTION
 *    Returns 0 if the error code taper_error matches expected,
 *    1 else. If returns 1, prints detailed error code.
 * RESULT
 *    int (exit code), 0 if ok.
 * NOTES
 * BUGS
 * TODO
 */
static int is_fine_taper(const char *what,
		         taper_error_t taper_error,
			 taper_error_t expected);

/* Function implementation */

int main(int argc, char **argv) {
   /* BUGS
    *    - Should have been done with getopt() in the first place.
    */
   if (argc >= 4) {
      if (!strcasecmp(argv[1], "read_part") && (argc == 7)) {
         return do_read_part(argv[0],
                             argv[2],
                             argv[3],
                             argv[4],
                             argv[5],
                             atoi(argv[6]));
      }
      else if (!strcasecmp(argv[1], "write_part") && (argc == 6)) {
         return do_write_part(argv[0],
			      argv[2],
			      argv[3],
			      argv[4],
			      argv[5]);
      }
      else if (!strcasecmp(argv[1], "show") && (argc == 4)) {
   	 return do_show(argv[0], argv[2], argv[3]);
      }
      else if (!strcasecmp(argv[1], "label") && (argc == 6)) {
         return do_label(argv[0], argv[2], argv[3], argv[4], atoi(argv[5]));
      }
      else if (!strcasecmp(argv[1], "load_tape") && (argc == 5)) {
         return do_load_tape(argv[0], argv[2], argv[3], argv[4]);
      }
      else if (!strcasecmp(argv[1], "unload_tape") && (argc == 4)) {
         return do_unload_tape(argv[0], argv[2], argv[3]);
      }
      else {
         return usage(argv[0]);
      }
   }
   else {
      return usage(argv[0]);
   }

   /* NOT REACHED */
}

/* Private function implementation */

static int usage(const char *program_name) {
   fprintf(stderr, "%s show changer_device_name tape_device_name\n"
                   "%s label changer_device_name tape_device_name label slot\n"
                   "%s load_tape changer_device_name tape_device_name label\n"
                   "%s unload_tape changer_device_name label tape_device_name\n"
                   "%s write_part changer_device_name tape_device_name label part_file_name\n"
                   "%s read_part changer_device_name tape_device_name label part_file_name file_num\n"
                   "%s: bad args.\n",
                   program_name,
                   program_name,
                   program_name,
                   program_name,
                   program_name,
                   program_name,
                   program_name);

   return 2;
}

static int do_show(const char *program_name,
                   const char *changer_device_name,
                   const char *tape_device_name) {
   changer_context_t changer_context;
   int result = 1; /* FAILED */

   debug_log(DEBUG_LEVEL_INFO,
             "%s: Starting scanning on device %s with drive %s.\n",
             program_name,
             changer_device_name,
             tape_device_name);

   changer_context = changer_initialize(changer_device_name);
   if (changer_context) {
      taper_context_t taper_context;

      debug_log(DEBUG_LEVEL_INFO, "Changer open.\n");

      taper_context = taper_initialize(tape_device_name,
                                       changer_context);
      if (taper_context) {
         debug_log(DEBUG_LEVEL_INFO, "Taper open.\n");      

	 result = is_fine_taper("scanning",
                                taper_load_tape(taper_context, NULL),
                                TAPER_ERROR_LABEL_NOT_KNOWN);

         taper_deinitialize(&taper_context);
      }
      else {
         debug_log(DEBUG_LEVEL_ERROR, "Can't initialize taper.\n");
      }

      changer_deinitialize(&changer_context);
   }
   else {
      debug_log(DEBUG_LEVEL_FATAL,
                "Can't initialize changer device %s.\n",
                changer_device_name);
   }

   return result;
}

static int do_label(const char *program_name,
                    const char *changer_device_name,
                    const char *tape_device_name,
                    const char *label,
                    const unsigned int slot) {
   int result = 1; /* FAILED */
   changer_context_t changer_context;

   changer_context = changer_initialize(changer_device_name);
   if (changer_context) {
      taper_context_t taper_context;

      debug_log(DEBUG_LEVEL_INFO, "Changer open.\n");

      taper_context = taper_initialize(tape_device_name,
                                       changer_context);
      if (taper_context) {
         debug_log(DEBUG_LEVEL_INFO, "Taper open.\n");      

	 result = is_fine_taper("label",
                                taper_label_tape(taper_context,
                                                 slot,
                                                 label,
                                                 FALSE),
                                TAPER_ERROR_NONE);

         taper_deinitialize(&taper_context);
      }
      else {
         debug_log(DEBUG_LEVEL_ERROR, "Can't initialize taper.\n");
      }

      changer_deinitialize(&changer_context);
   }
   else {
      debug_log(DEBUG_LEVEL_FATAL,
                "Can't initialize changer device %s.\n",
                changer_device_name);
   }

   return result;
}

static int do_load_tape(const char *program_name,
			const char *changer_device_name,
			const char *tape_device_name,
			const char *label) {
   int result = 1; /* FAILED */
   changer_context_t changer_context;

   changer_context = changer_initialize(changer_device_name);
   if (changer_context) {
      taper_context_t taper_context;

      changer_dump_element_status(changer_context);

      debug_log(DEBUG_LEVEL_INFO, "Changer open.\n");

      taper_context = taper_initialize(tape_device_name,
                                       changer_context);
      if (taper_context) {
         debug_log(DEBUG_LEVEL_INFO, "Taper open.\n");      

	 result = is_fine_taper("load tape",
                                taper_load_tape(taper_context, label),
		                TAPER_ERROR_NONE);

         taper_deinitialize(&taper_context);
      }
      else {
         debug_log(DEBUG_LEVEL_ERROR, "Can't initialize taper.\n");
      }


      changer_deinitialize(&changer_context);
   }
   else {
      debug_log(DEBUG_LEVEL_FATAL,
                "Can't initialize changer device %s.\n",
                changer_device_name);
   }

   return result;
}

static int do_unload_tape(const char *program_name,
		  	  const char *changer_device_name,
			  const char *tape_device_name) {
   int result = 1; /* FAILED */
   changer_context_t changer_context;

   changer_context = changer_initialize(changer_device_name);
   if (changer_context) {
      taper_context_t taper_context;

      changer_dump_element_status(changer_context);

      debug_log(DEBUG_LEVEL_INFO, "Changer open.\n");

      taper_context = taper_initialize(tape_device_name,
                                       changer_context);
      if (taper_context) {
         debug_log(DEBUG_LEVEL_INFO, "Taper open.\n");      

	 result = is_fine_taper("unload tape",
                                taper_unload_tape(taper_context),
		                TAPER_ERROR_LABEL_NOT_KNOWN);

         taper_deinitialize(&taper_context);
      }
      else {
         debug_log(DEBUG_LEVEL_ERROR, "Can't initialize taper.\n");
      }

      changer_deinitialize(&changer_context);
   }
   else {
      debug_log(DEBUG_LEVEL_FATAL,
                "Can't initialize changer device %s.\n",
                changer_device_name);
   }

   return result;
}

static int do_read_part(const char *program_name,
                        const char *changer_device_name,
                        const char *tape_device_name,
                        const char *label,
                        const char *output_file_name,
                        const taper_file_number_t file_number) {
   int result = 1; /* FAILED */
   changer_context_t changer_context;

   changer_context = changer_initialize(changer_device_name);
   if (changer_context) {
      taper_context_t taper_context;

      taper_context = taper_initialize(tape_device_name,
				       changer_context);
      if (taper_context) {
	 debug_log(DEBUG_LEVEL_INFO, "Taper open.\n");      

	 result = is_fine_taper("load tape",
                                taper_load_tape(taper_context, label),
		                TAPER_ERROR_NONE);

         if (result == 0) {
   	    debug_log(DEBUG_LEVEL_INFO, "Tape loaded.\n");

	    result = is_fine_taper("read part",
				   taper_read_part(taper_context,
						   output_file_name,
						   file_number),
				   TAPER_ERROR_NONE);
         }

	 taper_deinitialize(&taper_context);
      }
      else {
	 debug_log(DEBUG_LEVEL_ERROR, "Can't initialize taper.\n");
      }

      changer_deinitialize(&changer_context);
   }
   else {
      debug_log(DEBUG_LEVEL_FATAL,
                "Can't initialize changer device %s.\n",
                changer_device_name);
   }
   
   return result;
}
                        
static int do_write_part(const char *program_name,
                         const char *changer_device_name,
                         const char *tape_device_name,
                         const char *label,
                         const char *input_file_name) {
   int result = 1; /* FAILED */
   changer_context_t changer_context;

   changer_context = changer_initialize(changer_device_name);
   if (changer_context) {
      taper_context_t taper_context;

      taper_context = taper_initialize(tape_device_name,
				       changer_context);
      if (taper_context) {
	 taper_file_number_t file_number;
	 size_t residual;
	 debug_log(DEBUG_LEVEL_INFO, "Taper open.\n");      

	 result = is_fine_taper("load tape",
                                taper_load_tape(taper_context, label),
		                TAPER_ERROR_NONE);

         if (result == 0) {
   	    debug_log(DEBUG_LEVEL_INFO, "Tape loaded.\n");

	    result = is_fine_taper("write part",
				   taper_write_part(taper_context,
						    input_file_name,
						    &file_number,
						    &residual),
				   TAPER_ERROR_NONE);

	    if (result == 0) {
	       debug_log(DEBUG_LEVEL_INFO,
			 "Taper done: %u / %u.\n",
			 file_number,
			 residual);
	    }
         }

	 taper_deinitialize(&taper_context);
      }
      else {
	 debug_log(DEBUG_LEVEL_ERROR, "Can't initialize taper.\n");
      }

      changer_deinitialize(&changer_context);
   }
   else {
      debug_log(DEBUG_LEVEL_FATAL,
                "Can't initialize changer device %s.\n",
                changer_device_name);
   }
   
   return result;
}

static int is_fine_taper(const char *what,
			  taper_error_t taper_error,
			  taper_error_t expected) {
   if (taper_error == expected) {
      return 0;
   }
   else {
      debug_log(DEBUG_LEVEL_ERROR,
                "Error %d while %s: %s.\n",
                taper_error,
                what,
                taper_error_description(taper_error));      
      return 1;
   }
   /* NOT REACHED */
}
