/* Project filed -- Intelligent file cache and migration system
 * server_defs.h -- General definitions global definitions.
 * Marc SCHAEFER <schaefer@alphanet.ch>
 * Created: Sun Apr 9 2000
 * LICENSE
 *    (C) 2000 by Marc SCHAEFER, licensed under the terms of the
 *     GNU General Public License as published by the Free Software
 *     Foundation; either version 2 or later (at your option).
 * DESCRIPTION
 *    This file contains general global definitions.
 * NOTES
 * BUGS
 * TODO
 * BASED-ON
 *    Original work.
 * MODIFICATION-HISTORY
 * $Id: server_defs.h,v 1.8 2001/02/04 19:57:16 schaefer Exp $
 */

#ifndef _SERVER_DEFS_H
#define _SERVER_DEFS_H

/* All including final zero */
#define MAX_CLIENT_NAME_SIZE  30 /* for protocol */
#define MAX_SERVER_NAME_SIZE  30
#define MAX_FILE_NAME_SIZE   100
#define MAX_LABEL_NAME_SIZE   30

typedef unsigned int session_flags_t;
#define SESSION_FLAGS_TO_SIZE_GUESS 1
#define SESSION_FLAGS_TO_SIZE_EXACT 2 /* always exact for FROM */
#define SESSION_FLAGS_SLOW 4
#define SESSION_FLAGS_FAST 8
#define SESSION_FLAGS_TO_CONTIGUOUS 16
#define SESSION_FLAGS_TO_SAME_LOCATION 32
#define SESSION_FLAGS_TO_CONTIGUOUS_TO_REFERENCE 64
#define SESSION_FLAGS_TO_SAME_LOCATION_AS_REFERENCE 128
#define SESSION_FLAGS_TO_ENCRYPTED_DATA 256
#define SESSION_FLAGS_TO_ENCRYPTED_XMIT 512
#define SESSION_FLAGS_TO_KEEP_IF_ABORTED 1024
#define SESSION_FLAGS_TO_ERASE_WHOLE_FILE 2048

#define MAX_BLOCK_SIZE 131072 /* for I/O, namely filesend blocks, not chunks */

typedef unsigned long int block_size_t;
typedef unsigned long int block_count_t;
typedef unsigned long int block_offset_t;

typedef enum {
   DIRECTION_TO_CACHE,  /* client/tape to cache */
   DIRECTION_FROM_CACHE /* client/tape from cache */
} direction_t;

/* BUGS
 *    - This is also used as client->server communication,
 *      and thus there might be big endian/little endian
 *      problems. Maybe force the client to use little
 *      endian, or add another layer ?  This would
 *      typically be a case for SecureRPC or XDR.
 */
typedef struct {
   direction_t direction;

   char client_name[MAX_CLIENT_NAME_SIZE];
   char server_name[MAX_SERVER_NAME_SIZE];
   char file_name[MAX_FILE_NAME_SIZE];

   char reference_file_name[MAX_FILE_NAME_SIZE];

   session_flags_t flags;

   block_size_t block_size;
   block_count_t blocks; /* in dumper block size units */
   block_offset_t offset;
} session_dumper_info_t; /* client server */

#endif /* !_SERVER_DEFS_H */
